# automatically generated by the FlatBuffers compiler, do not modify

# namespace: FBOutput

import tdw.flatbuffers

class QuitSignal(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsQuitSignal(cls, buf, offset):
        n = tdw.flatbuffers.encode.Get(tdw.flatbuffers.packer.uoffset, buf, offset)
        x = QuitSignal()
        x.Init(buf, n + offset)
        return x

    # QuitSignal
    def Init(self, buf, pos):
        self._tab = tdw.flatbuffers.table.Table(buf, pos)

    # QuitSignal
    def Ok(self):
        o = tdw.flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return bool(self._tab.Get(tdw.flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

def QuitSignalStart(builder): builder.StartObject(1)
def QuitSignalAddOk(builder, ok): builder.PrependBoolSlot(0, ok, 0)
def QuitSignalEnd(builder): return builder.EndObject()
