# automatically generated by the FlatBuffers compiler, do not modify

# namespace: FBOutput

import tdw.flatbuffers

class MagnebotWheels(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsMagnebotWheels(cls, buf, offset):
        n = tdw.flatbuffers.encode.Get(tdw.flatbuffers.packer.uoffset, buf, offset)
        x = MagnebotWheels()
        x.Init(buf, n + offset)
        return x

    # MagnebotWheels
    def Init(self, buf, pos):
        self._tab = tdw.flatbuffers.table.Table(buf, pos)

    # MagnebotWheels
    def Id(self):
        o = tdw.flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(tdw.flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

    # MagnebotWheels
    def Success(self):
        o = tdw.flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return bool(self._tab.Get(tdw.flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

def MagnebotWheelsStart(builder): builder.StartObject(2)
def MagnebotWheelsAddId(builder, id): builder.PrependInt32Slot(0, id, 0)
def MagnebotWheelsAddSuccess(builder, success): builder.PrependBoolSlot(1, success, 0)
def MagnebotWheelsEnd(builder): return builder.EndObject()
